const fs = require('fs');
const path = require('path');
const process = require('process');

const correct_name = ['.','-','_','/','\\','!','@','#','$','%','^','&','*','(',')','+','=','~','`'];

let directory = __dirname;
let file = null;

// Prototypes - start

String.prototype.replaceAll = (search, replacement) => {
    var target = this;
    return target.replace(new RegExp(search, 'g'), replacement);
};

Object.prototype.length = () => {
    var target = this;
    return Object.keys(target).length
}

// Prototypes - end

// App functions - start

const app = {
    error : (error)=>{
        // Error handeling!        
        process.stderr.write(error.toString()+'\n');
        process.exit(1);
    },
    set : (name)=>{
        name = '.'+name+'.json'; // <- hide file and ext : .json
        return path.join(directory,name);        
    },
    create : async ()=>{                
        // if json file not exists , create
        if(await fs.existsSync(file) == false)
            //                 file dir ,    default data    , utf8 encoding
            await fs.writeFileSync(file , JSON.stringify({}) , {encoding : 'utf8'});
    },
    read : async ()=>{
        return new Promise( async (resolve,reject)=>{            
            // if file == null 
            if(file == null || file == '' || file == undefined || file == 'undefined') reject({ status : false , message : 'Database file not found!'});
            // if file not exists
            else if(await fs.existsSync(file) == false) reject({ status : false , message : 'Database file not exists!'});
            // else return data in file
            else {
                // read file and parse to JSON
                resolve(await JSON.parse( await fs.readFileSync(file,{ encoding : 'utf8' }) ));
            }
        })
    },
    write : async (data={})=>{
         // if json file not exists return error
         if(await fs.existsSync(file) == false) return Promise.reject({ status : false , message : 'Database file not exists!'});
         else await fs.writeFileSync(file , JSON.stringify(data) , {encoding : 'utf8'});
    }
};

// App functions - end

const main = class electron_json_db{
    constructor(name='',dir=directory){
        //If directory not __dirname
        if(dir != directory) directory = dir;
        // replaceing table name to correct name
        correct_name.forEach(async correct=>{
            if(name.includes(correct))
                name = name.replaceAll(correct,'');
        })
        
        // Chack file exists
        file = app.set(name);
    
        // Create file
        app.create();
    }
    /**
     * Insert data to database - Use : insert(name = ' ' , value = ' ')
     * 
     * JSON is key and value
     * @param {string} name - name or key in json
     * @param {*} value - value in json
     * @returns {object} status and message
     */
    async insert(name = '',value){
        if(typeof name != 'string') return { status : false , message : 'Typeof name must be String.' };
        try {
            let data = await app.read();
            if(data[name]) return { status : false , message : 'This name/key exists.' };
            else {
                data[name] = value;
                await app.write(data);
                return { status : true , message : 'Data inserted!' , data : { name : value }}
            }
        } catch (error) {
            return error;
        }
    }

    /**
     * Get data from database - Use : get() -> returns all data , get(name = ' ') -> return value of name
     * @param {string} name - name or key in json
     * @returns {object} status and message
     */
    async get(name=''){
        try {
            let data = await app.read();
            if(name == '') return { status : true , message : 'Returns all data' , data : data };
            else if(name != '') {
                if(!data[name]) return { status : false , message : `No value found for : ${name}` };
                else return { status : true , message : `Return data of ${name}` , data : data[name] };
            }
            else return { status : false , message : 'Error in get Function!'};
        } catch (error) {
            return error;
        }
    }

     /**
     * Update data in database - Use : update(name = ' ' , value = ' ')
     * 
     * @param {string} name - name or key in json
     * @param {*} value - value in json
     * @returns {object} status and message
     */
    async update(name='',value){
        if(typeof name != 'string') return { status : false , message : 'Typeof name must be String.' };
        try {
            let data = await app.read();
            if(!data[name]) return { status : false , message : `No value found for : ${name}` };
            else if(data[name] == value) return { status : false , message : 'Value has not changed.' };
            else {
                data[name] = value;
                await app.write(data);
                let object = new Object();
                object[name] = value;
                return { status : true , message : 'Data updated!' , data : object}
            }
        } catch (error) {
            return error;
        }   
    }

    /**
     * Remove data from database - Use : remove(name = ' ')
     * @param {string} name - name or key in json
     * @returns {object} status and message
     */
    async remove(name=''){
        if(typeof name != 'string') return { status : false , message : 'Typeof name must be String.' };
        try {
            let data = await app.read();
            if(!data[name]) return { status : false , message : `No value found for : ${name}` };
            else {
                delete data[name];
                await app.write(data);
                return { status : true , message : 'Data removed!'}
            }
        } catch (error) {
            return error;
        }
    }

    /**
     * Check exists data is in database - Use : exists(name = ' ')
     * @param {string} name - name or key in json
     * @returns {object} status and exists and message
     */
    async exists(name=''){
        if(typeof name != 'string') return { status : false , message : 'Typeof name must be String.' };
        try {
            let data = await app.read();
            if(!data[name]) return { status : true , exists : false };
            else return { status : true , exists : true };
        } catch (error) {
            return error;
        }
    }
}

module.exports = main;